/*
	App.cpp
	Copyright 1996-1997 Be Do Have. All Rights Reserved.
*/

#include <Alert.h>
#include <support/Debug.h>

#include <stdio.h>
#include <string.h>

extern "C" {
#include "civclient.h"
#include "clinet.h"
#include "config.h"
#include "fcintl.h"
#include "game.h"
#include "helpdata.h"
#include "log.h"
#include "mem.h"
#include "options.h"
#include "shared.h"
#include "tilespec.h"
#include "version.h"

/* STEALTH GLOBALS from client/civclient.c */
extern char     metaserver      [];
extern char     server_host     [];
extern char     name            [];
extern int		server_port;
/* STEALTH GLOBALS from gui-beos/gui_main.c */
extern char     logfile      [];
}

#include "App.hpp"
#include "Defs.hpp"
#include "MainWindow.hpp"
#include "graphics.hpp"
#include "Backend.hpp"

// SIGH -- globals 
App *app;


//--------------------------------------------------------------

App::App()
	: BdhApp( APP_SIGNATURE ), mainWin(0),
    	loglevel(LOG_NORMAL), tile_set_name(0)
{
	// set up full defaults here, as if no arguments given on command-line
	tile_set_name = "trident";

    server_port=DEFAULT_SOCK_PORT;
    strcpy(server_host, "localhost");
    strcpy(metaserver, METALIST_ADDR);
    name[0] = '\0';

	// @@@@
}


App::~App()
{
}


void
App::MessageReceived( BMessage *msg )
{
	switch (msg->what) {
	default:
		BdhApp::MessageReceived(msg);
		break;
	} // switch
}


void
App::ReadyToRun( void )
{
	BdhApp::ReadyToRun();

	// finish setup required before main window
	boot_help_texts();
	tilespec_read_toplevel(tile_set_name);
	tilespec_load_tiles();
	load_intro_gfx();
	load_cursors();
	load_options();
	set_client_state(CLIENT_PRE_GAME_STATE);

	// set up main window and go to it!
	mainWin = new MainWindow();
	if ( ! mainWin ) {
		BAlert *a = new BAlert( APP_NAME,
					"Could not construct main window!!", "ARGH!" );
		a->Go();
		PostMessage( B_QUIT_REQUESTED );
		return;
	}
	mainWin->Show();
}


// -- Receive command-line options and process them
//		(will not be called if no arguments given)
void
App::ArgvReceived(int32 argc, char **argv)
{
	// process all arguments given on command line
	// parse_options(argc, argv);
}


void
App::AboutRequested( void )
{
	BAlert *alert = new BAlert( APP_NAME,
		APP_NAME " " APP_VERSION_STRING " \n\n" \
		COPYRIGHT_STRING " \n" LICENSE_TYPE_TEXT " \n" \
		LICENSE_COPY1_TEXT " " LICENSE_COPY2_TEXT " \n" \
		LICENSE_THANKS_TEXT "\n\n" ABOUT_BOX_CREDITS_TEXT,
		"Primo!" );
	alert->Go();
}


bool
App::QuitRequested( void )
{
	return BdhApp::QuitRequested();
}


//============================================================================

// -- Start the visible application.
//	NOTE:  args get delivered via other mechanism, to App::ArgvReceived()
void
app_main( int argc, char** argv )
{
	// Start the backend, to communicate with the server and cache data
	backend = new Backend();

	// Start the UI itself
	app = new App();
	app->parse_options(argc, argv);
    app->Run();

	// Clean up
	if ( backend && backend->Lock() ) {
		backend->Quit();
	}
	delete backend;
    delete app;
	fprintf( stderr, "FreeCiv rules!\n" );
}


void
App::parse_options(int argc, char **argv)
{
  char *option=NULL; /* really use as a pointer */
  int i;

  i = 1;
  while (i < argc)
  {
    if (is_option("--help", argv[i]))
      {
      print_usage(argv[0]);
      exit(0);
      }
    else if (is_option("--version",argv[i]))
      {
      fprintf(stderr, "%s\n", FREECIV_NAME_VERSION);
      exit(0);
      }
    else if ((option = get_option("--log",argv,&i,argc)) != NULL)
		strcpy( logfile, option );
    else if ((option = get_option("--name",argv,&i,argc)) != NULL)
      strcpy(name, option);
    else if ((option = get_option("--metaserver",argv,&i,argc)) != NULL)
      strcpy(metaserver,option);
    else if ((option = get_option("--port",argv,&i,argc)) != NULL)
      server_port=atoi(option);
    else if ((option = get_option("--server",argv,&i,argc)) != NULL)
      strcpy(server_host, option);
    else if ((option = get_option("--debug",argv,&i,argc)) != NULL)
    {
      loglevel=log_parse_level_str(option);
      if (loglevel==-1) {
        exit(1);
      }
    }
    else if ((option = get_option("--tiles",argv,&i,argc)) != NULL)
      tile_set_name=option;
    i += 1;
  }

  log_init(logfile, loglevel, NULL);

  /* after log_init: */
  if (name[0] == '\0') {
    strcpy(name, user_username());
  }
}


void
App::print_usage(const char *argv0)
{
  fprintf(stderr, _("Usage: %s [option ...]\nValid options are:\n"), argv0);
  fprintf(stderr, _("  -h, --help\t\tPrint a summary of the options.\n"));
  fprintf(stderr, _("  -l, --log=FILE\tUse FILE as logfile.\n"));
  fprintf(stderr, _("  -N, --Name=NAME\tUse NAME.\n"));
  fprintf(stderr, _("  -p, --port=PORT\tConnect to PORT.\n"));
  fprintf(stderr, _("  -s, --server=SERVER\tConnect to the server SERVER.\n"));
  fprintf(stderr, _("  -m, --metaserver=METASERVER\tSet metaserver address to METASERVER.\n"));
#ifdef DEBUG
  fprintf(stderr, _("  -d, --debug=LEVEL\tSet debug log LEVEL (0,1,2,3,"
	                                          "or 3:file1,min,max:...)\n"));
#else
  fprintf(stderr, _("  -d, --debug=LEVEL\tSet debug log LEVEL (0,1,2).\n"));
#endif
  fprintf(stderr, _("  -t, --tiles=DIR\tLook in directory DIR for the tiles.\n"));
  fprintf(stderr, _("  -v, --version\t\tPrint the version number.\n"));
}

